@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2009 Fredrik Mellbin
+//  Copyright (c) 2007-2011 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -22,15 +22,8 @@
 #include "ffms.h"
 #include "avssources.h"
 #include "ffswscale.h"
-#include "ffpp.h"
 #include "avsutils.h"
 
-static int GetNumberOfLogicalCPUs() {
-	SYSTEM_INFO SI;
-	GetSystemInfo(&SI);
-	return SI.dwNumberOfProcessors;
-}
-
 static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
 	FFMS_Init((int)AvisynthToFFCPUFlags(Env->GetCPUFlags()),  Args[7].AsBool(false));
 
@@ -74,8 +67,8 @@ static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvi
 	else
 		Env->ThrowError("FFIndex: Invalid demuxer requested");
 
-	FFMS_Index *Index = NULL;
-	if (OverWrite || !(Index = FFMS_ReadIndex(CacheFile, &E))) {
+	FFMS_Index *Index = FFMS_ReadIndex(CacheFile, &E);
+	if (OverWrite || !Index || (Index && FFMS_IndexBelongsToFile(Index, Source, 0) != FFMS_ERROR_SUCCESS)) {
 		FFMS_Indexer *Indexer = FFMS_CreateIndexerWithDemuxer(Source, Demuxer, &E);
 		if (!Indexer)
 			Env->ThrowError("FFIndex: %s", E.Buffer);
@@ -113,15 +106,15 @@ static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScri
 	const char *CacheFile = Args[3].AsString("");
 	int FPSNum = Args[4].AsInt(-1);
 	int FPSDen = Args[5].AsInt(1);
-	const char *PP = Args[6].AsString("");
-	int Threads = Args[7].AsInt(-1);
-	const char *Timecodes = Args[8].AsString("");
-	int SeekMode = Args[9].AsInt(1);
-	int RFFMode = Args[10].AsInt(0);
-	int Width = Args[11].AsInt(0);
-	int Height = Args[12].AsInt(0);
-	const char *Resizer = Args[13].AsString("BICUBIC");
-	const char *ColorSpace = Args[14].AsString("");
+	int Threads = Args[6].AsInt(-1);
+	const char *Timecodes = Args[7].AsString("");
+	int SeekMode = Args[8].AsInt(1);
+	int RFFMode = Args[9].AsInt(0);
+	int Width = Args[10].AsInt(0);
+	int Height = Args[11].AsInt(0);
+	const char *Resizer = Args[12].AsString("BICUBIC");
+	const char *ColorSpace = Args[13].AsString("");
+	const char *VarPrefix = Args[15].AsString("");
 
 	if (FPSDen < 1)
 		Env->ThrowError("FFVideoSource: FPS denominator needs to be 1 or higher");
@@ -132,9 +125,6 @@ static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScri
 	if (SeekMode < -1 || SeekMode > 3)
 		Env->ThrowError("FFVideoSource: Invalid seekmode selected");
 
-	if (Threads <= 0) 
-		Threads = GetNumberOfLogicalCPUs();
-
 	if (RFFMode < 0 || RFFMode > 2)
 		Env->ThrowError("FFVideoSource: Invalid RFF mode selected");
 
@@ -189,13 +179,10 @@ static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScri
 		}
 	}
 
-	if (!strcmp(PP, ""))
-		PP = NULL;
-
 	AvisynthVideoSource *Filter;
 
 	try {
-		Filter = new AvisynthVideoSource(Source, Track, Index, FPSNum, FPSDen, PP, Threads, SeekMode, RFFMode, Width, Height, Resizer, ColorSpace, Env);
+		Filter = new AvisynthVideoSource(Source, Track, Index, FPSNum, FPSDen, Threads, SeekMode, RFFMode, Width, Height, Resizer, ColorSpace, VarPrefix, Env);
 	} catch (...) {
 		FFMS_DestroyIndex(Index);
 		throw;
@@ -221,6 +208,7 @@ static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScri
 	bool Cache = Args[2].AsBool(true);
 	const char *CacheFile = Args[3].AsString("");
 	int AdjustDelay = Args[4].AsInt(-1);
+	const char *VarPrefix = Args[6].AsString("");
 
 	if (Track <= -2)
 		Env->ThrowError("FFAudioSource: No audio track selected");
@@ -291,9 +279,9 @@ static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScri
 	AvisynthAudioSource *Filter;
 
 	try {
-		Filter = new AvisynthAudioSource(Source, Track, Index, AdjustDelay, Env);
+		Filter = new AvisynthAudioSource(Source, Track, Index, AdjustDelay, VarPrefix, Env);
 	} catch (...) {
-		FFMS_DestroyIndex(Index);	
+		FFMS_DestroyIndex(Index);
 		throw;
 	}
 
@@ -301,12 +289,6 @@ static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScri
 	return Filter;
 }
 
-#ifdef FFMS_USE_POSTPROC
-static AVSValue __cdecl CreateFFPP(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
-	return new FFPP(Args[0].AsClip(), Args[1].AsString(""), Env);
-}
-#endif // FFMS_USE_POSTPROC
-
 static AVSValue __cdecl CreateSWScale(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
 	return new SWScale(Args[0].AsClip(), Args[1].AsInt(0), Args[2].AsInt(0), Args[3].AsString("BICUBIC"), Args[4].AsString(""), Env);
 }
@@ -320,16 +302,19 @@ static AVSValue __cdecl FFSetLogLevel(AVSValue Args, void* UserData, IScriptEnvi
 	return FFMS_GetLogLevel();
 }
 
+static AVSValue __cdecl FFGetVersion(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
+	int Version = FFMS_GetVersion();
+	return Env->Sprintf("%d.%d.%d.%d", Version >> 24, (Version & 0xFF0000) >> 16, (Version & 0xFF00) >> 8, Version & 0xFF);
+}
+
 extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* Env) {
     Env->AddFunction("FFIndex", "[source]s[cachefile]s[indexmask]i[dumpmask]i[audiofile]s[errorhandling]i[overwrite]b[utf8]b[demuxer]s", CreateFFIndex, 0);
-	Env->AddFunction("FFVideoSource", "[source]s[track]i[cache]b[cachefile]s[fpsnum]i[fpsden]i[pp]s[threads]i[timecodes]s[seekmode]i[rffmode]i[width]i[height]i[resizer]s[colorspace]s[utf8]b", CreateFFVideoSource, 0);
-    Env->AddFunction("FFAudioSource", "[source]s[track]i[cache]b[cachefile]s[adjustdelay]i[utf8]b", CreateFFAudioSource, 0);
-#ifdef FFMS_USE_POSTPROC
-	Env->AddFunction("FFPP", "c[pp]s", CreateFFPP, 0);
-#endif // FFMS_USE_POSTPROC
+	Env->AddFunction("FFVideoSource", "[source]s[track]i[cache]b[cachefile]s[fpsnum]i[fpsden]i[threads]i[timecodes]s[seekmode]i[rffmode]i[width]i[height]i[resizer]s[colorspace]s[utf8]b[varprefix]s", CreateFFVideoSource, 0);
+	Env->AddFunction("FFAudioSource", "[source]s[track]i[cache]b[cachefile]s[adjustdelay]i[utf8]b[varprefix]s", CreateFFAudioSource, 0);
 	Env->AddFunction("SWScale", "c[width]i[height]i[resizer]s[colorspace]s", CreateSWScale, 0);
 	Env->AddFunction("FFGetLogLevel", "", FFGetLogLevel, 0);
 	Env->AddFunction("FFSetLogLevel", "i", FFSetLogLevel, 0);
+	Env->AddFunction("FFGetVersion", "", FFGetVersion, 0);
 
     return "FFmpegSource - The Second Coming V2.0 Final";
 }
