@@ -45,17 +45,8 @@ enum { AVISYNTH_INTERFACE_VERSION = 3 };
 /* Define all types necessary for interfacing with avisynth.dll
    Moved from internal.h */
 
-// Win32 API macros, notably the types BYTE, DWORD, ULONG, etc. 
-#include <windef.h>  
-
-#if (defined(_WIN64) && (_MSC_VER >= 1400))
-extern "C" LONG __cdecl _InterlockedIncrement(LONG volatile * pn);
-extern "C" LONG __cdecl _InterlockedDecrement(LONG volatile * pn);
-#pragma intrinsic(_InterlockedIncrement)
-#pragma intrinsic(_InterlockedDecrement)
-#define InterlockedIncrement _InterlockedIncrement
-#define InterlockedDecrement _InterlockedDecrement
-#endif
+// Win32 API macros, notably the types BYTE, DWORD, ULONG, etc.
+#include <windef.h>
 
 // COM interface macros
 #include <objbase.h>
@@ -88,7 +79,7 @@ typedef	long			PixOffset;
   #define _RPT2(a,b,c,d) ((void)0)
   #define _RPT3(a,b,c,d,e) ((void)0)
   #define _RPT4(a,b,c,d,e,f) ((void)0)
-  
+
   #define _ASSERTE(x) assert(x)
   #include <assert.h>
 #endif
@@ -110,7 +101,7 @@ typedef	long			PixOffset;
 typedef float SFLOAT;
 
 enum {SAMPLE_INT8  = 1<<0,
-        SAMPLE_INT16 = 1<<1, 
+        SAMPLE_INT16 = 1<<1,
         SAMPLE_INT24 = 1<<2,    // Int24 is a very stupid thing to code, but it's supported by some hardware.
         SAMPLE_INT32 = 1<<3,
         SAMPLE_FLOAT = 1<<4};
@@ -133,7 +124,7 @@ struct VideoInfo {
 
   // Colorspace properties.
   enum {
-    CS_BGR = 1<<28,  
+    CS_BGR = 1<<28,
     CS_YUV = 1<<29,
     CS_INTERLEAVED = 1<<30,
     CS_PLANAR = 1<<31
@@ -149,7 +140,7 @@ struct VideoInfo {
          CS_IYUV = 1<<4 | CS_YUV | CS_PLANAR  // same as above
          };
   int pixel_type;                // changed to int as of 2.5
-  
+
 
   int audio_samples_per_second;   // 0 means no audio
   int sample_type;                // as of 2.5
@@ -173,7 +164,7 @@ struct VideoInfo {
   bool IsRGB24() const { return (pixel_type&CS_BGR24)==CS_BGR24; } // Clear out additional properties
   bool IsRGB32() const { return (pixel_type & CS_BGR32) == CS_BGR32 ; }
   bool IsYUV() const { return !!(pixel_type&CS_YUV ); }
-  bool IsYUY2() const { return (pixel_type & CS_YUY2) == CS_YUY2; }  
+  bool IsYUY2() const { return (pixel_type & CS_YUY2) == CS_YUY2; }
   bool IsYV12() const { return ((pixel_type & CS_YV12) == CS_YV12)||((pixel_type & CS_I420) == CS_I420); }
   bool IsColorSpace(int c_space) const { return ((pixel_type & c_space) == c_space); }
   bool Is(int property) const { return ((pixel_type & property)==property ); }
@@ -182,8 +173,8 @@ struct VideoInfo {
   bool IsParityKnown() const { return ((image_type & IT_FIELDBASED)&&(image_type & (IT_BFF|IT_TFF))); }
   bool IsBFF() const { return !!(image_type & IT_BFF); }
   bool IsTFF() const { return !!(image_type & IT_TFF); }
-  
-  bool IsVPlaneFirst() const {return ((pixel_type & CS_YV12) == CS_YV12); }  // Don't use this 
+
+  bool IsVPlaneFirst() const {return ((pixel_type & CS_YV12) == CS_YV12); }  // Don't use this
   int BytesFromPixels(int pixels) const { return pixels * (BitsPerPixel()>>3); }   // Will not work on planar images, but will return only luma planes
   int RowSize() const { return BytesFromPixels(width); }  // Also only returns first plane on planar images
   int BMPSize() const { if (IsPlanar()) {int p = height * ((RowSize()+3) & ~3); p+=p>>1; return p;  } return height * ((RowSize()+3) & ~3); }
@@ -200,7 +191,7 @@ struct VideoInfo {
   void Set(int property)  { image_type|=property; }
   void Clear(int property)  { image_type&=~property; }
 
-  int BitsPerPixel() const { 
+  int BitsPerPixel() const {
     switch (pixel_type) {
       case CS_BGR24:
         return 24;
@@ -215,7 +206,7 @@ struct VideoInfo {
         return 0;
     }
   }
-  int BytesPerChannelSample() const { 
+  int BytesPerChannelSample() const {
     switch (sample_type) {
     case SAMPLE_INT8:
       return sizeof(signed char);
@@ -263,11 +254,7 @@ struct VideoInfo {
 
 	unsigned __int64 temp = numerator | denominator; // Just looking top bit
 	unsigned u = 0;
-#ifdef _WIN64
-	while (temp & 0xffffffff80000000ull) { // or perhaps > 16777216*2
-#else
 	while (temp & 0xffffffff80000000) { // or perhaps > 16777216*2
-#endif
 	  temp = Int64ShrlMod32(temp, 1);
 	  u++;
 	}
@@ -305,23 +292,12 @@ class VideoFrameBuffer {
   const int data_size;
   // sequence_number is incremented every time the buffer is changed, so
   // that stale views can tell they're no longer valid.
-#ifdef _WIN64
-  volatile long sequence_number;
-#else
   long sequence_number;
-#endif
 
   friend class VideoFrame;
   friend class Cache;
-#ifdef _WIN64
-  friend class CacheMT;
-#endif
   friend class ScriptEnvironment;
-#ifdef _WIN64
-  volatile long refcount;
-#else
   long refcount;
-#endif
 
 public:
   VideoFrameBuffer(int size);
@@ -329,11 +305,7 @@ public:
   ~VideoFrameBuffer();
 
   const BYTE* GetReadPtr() const { return data; }
-#ifdef _WIN64
-  BYTE* GetWritePtr() { InterlockedIncrement(&sequence_number); return data; }
-#else
   BYTE* GetWritePtr() { ++sequence_number; return data; }
-#endif
   int GetDataSize() { return data_size; }
   int GetSequenceNumber() { return sequence_number; }
   int GetRefcount() { return refcount; }
@@ -351,60 +323,40 @@ class AVSValue;
 // is overloaded to recycle class instances.
 
 class VideoFrame {
-#ifdef _WIN64
-  volatile long refcount;
-#else
   int refcount;
-#endif
   VideoFrameBuffer* const vfb;
   const int offset, pitch, row_size, height, offsetU, offsetV, pitchUV;  // U&V offsets are from top of picture.
 
   friend class PVideoFrame;
-#ifdef _WIN64
-  void AddRef() { InterlockedIncrement(&refcount); }
-  void Release() { VideoFrameBuffer* vfb_local = vfb; if (!InterlockedDecrement(&refcount)) InterlockedDecrement(&vfb_local->refcount); }
-#else
   void AddRef() { InterlockedIncrement((long *)&refcount); }
   void Release() { if (refcount==1) InterlockedDecrement(&vfb->refcount); InterlockedDecrement((long *)&refcount); }
-#endif
 
   friend class ScriptEnvironment;
   friend class Cache;
-#ifdef _WIN64
-  friend class CacheMT;
-#endif
 
   VideoFrame(VideoFrameBuffer* _vfb, int _offset, int _pitch, int _row_size, int _height);
   VideoFrame(VideoFrameBuffer* _vfb, int _offset, int _pitch, int _row_size, int _height, int _offsetU, int _offsetV, int _pitchUV);
 
-#ifdef _WIN64
-  void* operator new (size_t size);
-#else
-  void* operator new(unsigned size);
-#endif
+  void* operator new(size_t size);
 // TESTME: OFFSET U/V may be switched to what could be expected from AVI standard!
 public:
   int GetPitch() const { return pitch; }
   int GetPitch(int plane) const { switch (plane) {case PLANAR_U: case PLANAR_V: return pitchUV;} return pitch; }
   int GetRowSize() const { return row_size; }
-#ifdef _WIN64
-  __declspec(noinline) int GetRowSize(int plane) const {
-#else
-  int GetRowSize(int plane) const { 
-#endif
+  int GetRowSize(int plane) const {
     switch (plane) {
     case PLANAR_U: case PLANAR_V: if (pitchUV) return row_size>>1; else return 0;
-    case PLANAR_U_ALIGNED: case PLANAR_V_ALIGNED: 
-      if (pitchUV) { 
+    case PLANAR_U_ALIGNED: case PLANAR_V_ALIGNED:
+      if (pitchUV) {
         int r = ((row_size+FRAME_ALIGN-1)&(~(FRAME_ALIGN-1)) )>>1; // Aligned rowsize
-        if (r<=pitchUV) 
-          return r; 
-        return row_size>>1; 
+        if (r<=pitchUV)
+          return r;
+        return row_size>>1;
       } else return 0;
     case PLANAR_Y_ALIGNED:
       int r = (row_size+FRAME_ALIGN-1)&(~(FRAME_ALIGN-1)); // Aligned rowsize
-      if (r<=pitch) 
-        return r; 
+      if (r<=pitch)
+        return r;
       return row_size;
     }
     return row_size; }
@@ -445,11 +397,7 @@ public:
     return vfb->data + GetOffset(plane);
   }
 
-#ifdef _WIN64
-  ~VideoFrame() { VideoFrameBuffer* vfb_local = vfb; if (InterlockedDecrement(&refcount) >= 0) InterlockedDecrement(&vfb_local->refcount); }
-#else
   ~VideoFrame() { InterlockedDecrement(&vfb->refcount); }
-#endif
 };
 
 enum {
@@ -457,42 +405,27 @@ enum {
   CACHE_RANGE=1,
   CACHE_ALL=2,
   CACHE_AUDIO=3,
-#ifdef _WIN64
-  CACHE_AUDIO_NONE=4,
-  CACHE_AUDIO_AUTO=5
-#else
   CACHE_AUDIO_NONE=4
-#endif
  };
 
 // Base class for all filters.
 class IClip {
   friend class PClip;
   friend class AVSValue;
-#ifdef _WIN64
-  volatile long refcnt;
-  void AddRef() { InterlockedIncrement(&refcnt); }
-  void Release() { if (!InterlockedDecrement(&refcnt)) delete this; }
-#else
   int refcnt;
   void AddRef() { InterlockedIncrement((long *)&refcnt); }
   void Release() { InterlockedDecrement((long *)&refcnt); if (!refcnt) delete this; }
-#endif
 public:
   IClip() : refcnt(0) {}
 
   virtual int __stdcall GetVersion() { return AVISYNTH_INTERFACE_VERSION; }
-  
+
   virtual PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) = 0;
   virtual bool __stdcall GetParity(int n) = 0;  // return field parity if field_based, else parity of first field in frame
   virtual void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) = 0;  // start and count are in samples
   virtual void __stdcall SetCacheHints(int cachehints,int frame_range) = 0 ;  // We do not pass cache requests upwards, only to the next filter.
   virtual const VideoInfo& __stdcall GetVideoInfo() = 0;
-#ifdef _WIN64
-  virtual ~IClip() {}
-#else
   virtual __stdcall ~IClip() {}
-#endif
 };
 
 
@@ -596,8 +529,8 @@ public:
 
   PClip AsClip() const { _ASSERTE(IsClip()); return IsClip()?clip:0; }
   bool AsBool() const { _ASSERTE(IsBool()); return boolean; }
-  int AsInt() const { _ASSERTE(IsInt()); return integer; }   
-//  int AsLong() const { _ASSERTE(IsLong()); return longlong; } 
+  int AsInt() const { _ASSERTE(IsInt()); return integer; }
+//  int AsLong() const { _ASSERTE(IsLong()); return longlong; }
   const char* AsString() const { _ASSERTE(IsString()); return IsString()?string:0; }
   double AsFloat() const { _ASSERTE(IsFloat()); return IsInt()?integer:floating_pt; }
 
@@ -633,14 +566,8 @@ private:
     if (!init && IsClip() && clip)
       clip->Release();
     // make sure this copies the whole struct!
-#ifdef _WIN64
-    this->clip = src->clip;
-    this->type = src->type;
-    this->array_size = src->array_size;
-#else
     ((__int32*)this)[0] = ((__int32*)src)[0];
     ((__int32*)this)[1] = ((__int32*)src)[1];
-#endif
   }
 };
 
@@ -671,7 +598,7 @@ public:
 
 /* Helper classes useful to plugin authors */
 
-class AlignPlanar : public GenericVideoFilter 
+class AlignPlanar : public GenericVideoFilter
 {
 public:
   AlignPlanar(PClip _clip);
@@ -681,7 +608,7 @@ public:
 
 
 
-class FillBorder : public GenericVideoFilter 
+class FillBorder : public GenericVideoFilter
 {
 public:
   FillBorder(PClip _clip);
@@ -691,7 +618,7 @@ public:
 
 
 
-class ConvertAudio : public GenericVideoFilter 
+class ConvertAudio : public GenericVideoFilter
 /**
   * Helper class to convert audio to any format
  **/
@@ -707,9 +634,6 @@ public:
   static AVSValue __cdecl Create_24bit(AVSValue args, void*, IScriptEnvironment*);
   static AVSValue __cdecl Create_16bit(AVSValue args, void*, IScriptEnvironment*);
   static AVSValue __cdecl Create_8bit(AVSValue args, void*, IScriptEnvironment*);
-#ifdef _WIN64
-  static AVSValue __cdecl Create_Any  (AVSValue args, void*, IScriptEnvironment*);
-#endif
   virtual ~ConvertAudio();
 
 private:
@@ -737,7 +661,7 @@ private:
 
 
 // For GetCPUFlags.  These are backwards-compatible with those in VirtualDub.
-enum {                    
+enum {
                     /* slowest CPU to support extension */
   CPUF_FORCE			  = 0x01,   // N/A
   CPUF_FPU			    = 0x02,   // 386/486DX
@@ -750,28 +674,15 @@ enum {
   CPUF_X86_64       = 0xA0,   // Hammer (note: equiv. to 3DNow + SSE2, which only Hammer
                               //         will have anyway)
   CPUF_SSE3		= 0x100,		    // Some P4 & Athlon 64.
-#ifdef _WIN64
-  CPUF_SSSE3		= 0x200,   //  prescott?
-  CPUF_SSE4			= 0x400,   //  penryn	
-  CPUF_SSE4_2		= 0x800	   //  Core iX	
-#endif
 };
-#ifdef _WIN64
-#define MAX_INT MAXINT32
-#define MIN_INT MININT32  // ::FIXME:: research why this is not 0x80000000
-class IClipLocalStorage;
-#else
 #define MAX_INT 0x7fffffff
 #define MIN_INT -0x7fffffff
-#endif
+
+
 
 class IScriptEnvironment {
 public:
-#ifdef _WIN64
-  virtual ~IScriptEnvironment() {}
-#else
   virtual __stdcall ~IScriptEnvironment() {}
-#endif
 
   virtual /*static*/ long __stdcall GetCPUFlags() = 0;
 
@@ -825,15 +736,6 @@ public:
   virtual bool __stdcall PlanarChromaAlignment(PlanarChromaAlignmentMode key) = 0;
 
   virtual PVideoFrame __stdcall SubframePlanar(PVideoFrame src, int rel_offset, int new_pitch, int new_row_size, int new_height, int rel_offsetU, int rel_offsetV, int new_pitchUV) = 0;
-#ifdef _WIN64
-  virtual void __stdcall SetMTMode(int mode,int threads,bool temporary)=0;
-  virtual int __stdcall  GetMTMode(bool return_nthreads)=0;
-
-  virtual IClipLocalStorage* __stdcall AllocClipLocalStorage()=0;
-
-  virtual void __stdcall SaveClipLocalStorage()=0;
-  virtual void __stdcall RestoreClipLocalStorage()=0;
-#endif
 };
 
 
