@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2009 Fredrik Mellbin
+//  Copyright (c) 2007-2011 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -20,19 +20,21 @@
 
 #include "avssources.h"
 #include "avsutils.h"
+#include <libavutil/common.h>
 #include <cmath>
 
 
 
 AvisynthVideoSource::AvisynthVideoSource(const char *SourceFile, int Track, FFMS_Index *Index,
-		int FPSNum, int FPSDen, const char *PP, int Threads, int SeekMode, int RFFMode,
+		int FPSNum, int FPSDen, int Threads, int SeekMode, int RFFMode,
 		int ResizeToWidth, int ResizeToHeight, const char *ResizerName,
-		const char *ConvertToFormatName, IScriptEnvironment* Env) {
+		const char *ConvertToFormatName, const char *VarPrefix, IScriptEnvironment* Env) {
 
 	memset(&VI, 0, sizeof(VI));
 	this->FPSNum = FPSNum;
 	this->FPSDen = FPSDen;
 	this->RFFMode = RFFMode;
+	this->VarPrefix = VarPrefix;
 
 	char ErrorMsg[1024];
 	FFMS_ErrorInfo E;
@@ -43,11 +45,6 @@ AvisynthVideoSource::AvisynthVideoSource(const char *SourceFile, int Track, FFMS
 	if (!V)
 		Env->ThrowError("FFVideoSource: %s", E.Buffer);
 
-	if (PP && FFMS_SetPP(V, PP, &E)) {
-		FFMS_DestroyVideoSource(V);
-		Env->ThrowError("FFVideoSource: %s", E.Buffer);
-	}
-
 	try {
 		InitOutputFormat(ResizeToWidth, ResizeToHeight, ResizerName, ConvertToFormatName, Env);
 	} catch (AvisynthError &) {
@@ -94,16 +91,16 @@ AvisynthVideoSource::AvisynthVideoSource(const char *SourceFile, int Track, FFMS
 		for (int i = 0; i < VP->NumFrames; i++) {
 			int RepeatPict = FFMS_GetFrameInfo(VTrack, i)->RepeatPict;
 			int RepeatFields = ((RepeatPict + 1) * 2) / (RepeatMin + 1);
-			
+
 			for (int j = 0; j < RepeatFields; j++) {
 				if ((DestField + (VP->TopFieldFirst ? 0 : 1)) & 1)
 					FieldList[DestField / 2].Top = i;
-				else 
+				else
 					FieldList[DestField / 2].Bottom = i;
 				DestField++;
 			}
 		}
-		
+
 		if (RFFMode == 2) {
 			VI.num_frames = (VI.num_frames * 4) / 5;
 			VI.fps_denominator *= 5;
@@ -159,20 +156,18 @@ AvisynthVideoSource::AvisynthVideoSource(const char *SourceFile, int Track, FFMS
 	}
 
 	// Set AR variables
-	Env->SetVar("FFSAR_NUM", VP->SARNum);
-	Env->SetVar("FFSAR_DEN", VP->SARDen);
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFSAR_NUM"), VP->SARNum);
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFSAR_DEN"), VP->SARDen);
 	if (VP->SARNum > 0 && VP->SARDen > 0)
-		Env->SetVar("FFSAR", VP->SARNum / (double)VP->SARDen);
+		Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFSAR"), VP->SARNum / (double)VP->SARDen);
 
 	// Set crop variables
-	Env->SetVar("FFCROP_LEFT", VP->CropLeft);
-	Env->SetVar("FFCROP_RIGHT", VP->CropRight);
-	Env->SetVar("FFCROP_TOP", VP->CropTop);
-	Env->SetVar("FFCROP_BOTTOM", VP->CropBottom);
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFCROP_LEFT"), VP->CropLeft);
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFCROP_RIGHT"), VP->CropRight);
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFCROP_TOP"), VP->CropTop);
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFCROP_BOTTOM"), VP->CropBottom);
 
-	// Set color information
-	Env->SetVar("FFCOLOR_SPACE", VP->ColorSpace);
-	Env->SetVar("FFCOLOR_RANGE", VP->ColorRange);
+	Env->SetGlobalVar("FFVAR_PREFIX", this->VarPrefix);
 }
 
 AvisynthVideoSource::~AvisynthVideoSource() {
@@ -193,17 +188,21 @@ void AvisynthVideoSource::InitOutputFormat(
 	if (!F)
 		Env->ThrowError("FFVideoSource: %s", E.Buffer);
 
-	int64_t TargetFormats = (1 << FFMS_GetPixFmt("yuvj420p")) |
-		(1 << FFMS_GetPixFmt("yuv420p")) | (1 << FFMS_GetPixFmt("yuyv422")) |
-		(1 << FFMS_GetPixFmt("rgb32")) | (1 << FFMS_GetPixFmt("bgr24"));
+	int TargetFormats[4];
+	TargetFormats[0] = FFMS_GetPixFmt("yuv420p");
+	TargetFormats[1] = FFMS_GetPixFmt("yuyv422");
+	TargetFormats[2] = FFMS_GetPixFmt("bgra");
+	TargetFormats[3] = -1;
 
-	// PIX_FMT_NV12 is misused as a return value different to the defined ones in the function
-	PixelFormat TargetPixelFormat = CSNameToPIXFMT(ConvertToFormatName, PIX_FMT_NV12);
+	// PIX_FMT_NV21 is misused as a return value different to the defined ones in the function
+	PixelFormat TargetPixelFormat = CSNameToPIXFMT(ConvertToFormatName, PIX_FMT_NV21);
 	if (TargetPixelFormat == PIX_FMT_NONE)
 		Env->ThrowError("FFVideoSource: Invalid colorspace name specified");
 
-	if (TargetPixelFormat != PIX_FMT_NV12)
-		TargetFormats = static_cast<int64_t>(1) << TargetPixelFormat;
+	if (TargetPixelFormat != PIX_FMT_NV21) {
+		TargetFormats[0] = TargetPixelFormat;
+		TargetFormats[1] = -1;
+	}
 
 	if (ResizeToWidth <= 0)
 		ResizeToWidth = F->EncodedWidth;
@@ -215,14 +214,16 @@ void AvisynthVideoSource::InitOutputFormat(
 	if (Resizer == 0)
 		Env->ThrowError("FFVideoSource: Invalid resizer name specified");
 
-	if (FFMS_SetOutputFormatV(V, TargetFormats,
+	if (FFMS_SetOutputFormatV2(V, TargetFormats,
 		ResizeToWidth, ResizeToHeight, Resizer, &E))
 		Env->ThrowError("FFVideoSource: No suitable output format found");
 
 	F = FFMS_GetFrame(V, 0, &E);
+	TargetFormats[0] = F->ConvertedPixelFormat;
+	TargetFormats[1] = -1;
 
-	// This trick is required to first get the "best" default format and then set only that format as the output
-	if (FFMS_SetOutputFormatV(V, static_cast<int64_t>(1) << F->ConvertedPixelFormat,
+		// This trick is required to first get the "best" default format and then set only that format as the output
+	if (FFMS_SetOutputFormatV2(V, TargetFormats,
 		ResizeToWidth, ResizeToHeight, Resizer, &E))
 		Env->ThrowError("FFVideoSource: No suitable output format found");
 
@@ -242,9 +243,13 @@ void AvisynthVideoSource::InitOutputFormat(
 	if (RFFMode > 0 && ResizeToHeight != F->EncodedHeight)
 		Env->ThrowError("FFVideoSource: Vertical scaling not allowed in RFF mode");
 
-	if (RFFMode > 0 && 	TargetPixelFormat != PIX_FMT_NV12)
+	if (RFFMode > 0 && TargetPixelFormat != PIX_FMT_NV21)
 		Env->ThrowError("FFVideoSource: Only the default output colorspace can be used in RFF mode");
 
+	// set color information variables
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFCOLOR_SPACE"), F->ColorSpace);
+	Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFCOLOR_RANGE"), F->ColorRange);
+
 	if (VP->TopFieldFirst)
 		VI.image_type = VideoInfo::IT_TFF;
 	else
@@ -270,39 +275,39 @@ void AvisynthVideoSource::InitOutputFormat(
 
 void AvisynthVideoSource::OutputFrame(const FFMS_Frame *Frame, PVideoFrame &Dst, IScriptEnvironment *Env) {
 	if (VI.pixel_type == VideoInfo::CS_I420) {
-		Env->BitBlt(Dst->GetWritePtr(PLANAR_Y), Dst->GetPitch(PLANAR_Y), Frame->Data[0], Frame->Linesize[0], Dst->GetRowSize(PLANAR_Y), Dst->GetHeight(PLANAR_Y)); 
-		Env->BitBlt(Dst->GetWritePtr(PLANAR_U), Dst->GetPitch(PLANAR_U), Frame->Data[1], Frame->Linesize[1], Dst->GetRowSize(PLANAR_U), Dst->GetHeight(PLANAR_U)); 
-		Env->BitBlt(Dst->GetWritePtr(PLANAR_V), Dst->GetPitch(PLANAR_V), Frame->Data[2], Frame->Linesize[2], Dst->GetRowSize(PLANAR_V), Dst->GetHeight(PLANAR_V)); 
+		Env->BitBlt(Dst->GetWritePtr(PLANAR_Y), Dst->GetPitch(PLANAR_Y), Frame->Data[0], Frame->Linesize[0], Dst->GetRowSize(PLANAR_Y), Dst->GetHeight(PLANAR_Y));
+		Env->BitBlt(Dst->GetWritePtr(PLANAR_U), Dst->GetPitch(PLANAR_U), Frame->Data[1], Frame->Linesize[1], Dst->GetRowSize(PLANAR_U), Dst->GetHeight(PLANAR_U));
+		Env->BitBlt(Dst->GetWritePtr(PLANAR_V), Dst->GetPitch(PLANAR_V), Frame->Data[2], Frame->Linesize[2], Dst->GetRowSize(PLANAR_V), Dst->GetHeight(PLANAR_V));
 	} else if (VI.IsYUY2()) {
-		Env->BitBlt(Dst->GetWritePtr(), Dst->GetPitch(), Frame->Data[0], Frame->Linesize[0], Dst->GetRowSize(), Dst->GetHeight()); 
+		Env->BitBlt(Dst->GetWritePtr(), Dst->GetPitch(), Frame->Data[0], Frame->Linesize[0], Dst->GetRowSize(), Dst->GetHeight());
 	} else { // RGB
-		Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 1), -Dst->GetPitch(), Frame->Data[0], Frame->Linesize[0], Dst->GetRowSize(), Dst->GetHeight()); 
+		Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 1), -Dst->GetPitch(), Frame->Data[0], Frame->Linesize[0], Dst->GetRowSize(), Dst->GetHeight());
 	}
 }
 
 void AvisynthVideoSource::OutputField(const FFMS_Frame *Frame, PVideoFrame &Dst, int Field, IScriptEnvironment *Env) {
 	const FFMS_Frame *SrcPicture = (Frame);
-	
+
 	if (VI.pixel_type == VideoInfo::CS_I420) {
 		if (Field) {
-			Env->BitBlt(Dst->GetWritePtr(PLANAR_Y), Dst->GetPitch(PLANAR_Y) * 2, SrcPicture->Data[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(PLANAR_Y), Dst->GetHeight(PLANAR_Y) / 2); 
-			Env->BitBlt(Dst->GetWritePtr(PLANAR_U), Dst->GetPitch(PLANAR_U) * 2, SrcPicture->Data[1], SrcPicture->Linesize[1] * 2, Dst->GetRowSize(PLANAR_U), Dst->GetHeight(PLANAR_U) / 2); 
-			Env->BitBlt(Dst->GetWritePtr(PLANAR_V), Dst->GetPitch(PLANAR_V) * 2, SrcPicture->Data[2], SrcPicture->Linesize[2] * 2, Dst->GetRowSize(PLANAR_V), Dst->GetHeight(PLANAR_V) / 2); 
+			Env->BitBlt(Dst->GetWritePtr(PLANAR_Y), Dst->GetPitch(PLANAR_Y) * 2, SrcPicture->Data[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(PLANAR_Y), Dst->GetHeight(PLANAR_Y) / 2);
+			Env->BitBlt(Dst->GetWritePtr(PLANAR_U), Dst->GetPitch(PLANAR_U) * 2, SrcPicture->Data[1], SrcPicture->Linesize[1] * 2, Dst->GetRowSize(PLANAR_U), Dst->GetHeight(PLANAR_U) / 2);
+			Env->BitBlt(Dst->GetWritePtr(PLANAR_V), Dst->GetPitch(PLANAR_V) * 2, SrcPicture->Data[2], SrcPicture->Linesize[2] * 2, Dst->GetRowSize(PLANAR_V), Dst->GetHeight(PLANAR_V) / 2);
 		} else {
-			Env->BitBlt(Dst->GetWritePtr(PLANAR_Y) + Dst->GetPitch(PLANAR_Y), Dst->GetPitch(PLANAR_Y) * 2, SrcPicture->Data[0] + SrcPicture->Linesize[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(PLANAR_Y), Dst->GetHeight(PLANAR_Y) / 2); 
-			Env->BitBlt(Dst->GetWritePtr(PLANAR_U) + Dst->GetPitch(PLANAR_U), Dst->GetPitch(PLANAR_U) * 2, SrcPicture->Data[1] + SrcPicture->Linesize[1], SrcPicture->Linesize[1] * 2, Dst->GetRowSize(PLANAR_U), Dst->GetHeight(PLANAR_U) / 2); 
-			Env->BitBlt(Dst->GetWritePtr(PLANAR_V) + Dst->GetPitch(PLANAR_V), Dst->GetPitch(PLANAR_V) * 2, SrcPicture->Data[2] + SrcPicture->Linesize[2], SrcPicture->Linesize[2] * 2, Dst->GetRowSize(PLANAR_V), Dst->GetHeight(PLANAR_V) / 2); 
+			Env->BitBlt(Dst->GetWritePtr(PLANAR_Y) + Dst->GetPitch(PLANAR_Y), Dst->GetPitch(PLANAR_Y) * 2, SrcPicture->Data[0] + SrcPicture->Linesize[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(PLANAR_Y), Dst->GetHeight(PLANAR_Y) / 2);
+			Env->BitBlt(Dst->GetWritePtr(PLANAR_U) + Dst->GetPitch(PLANAR_U), Dst->GetPitch(PLANAR_U) * 2, SrcPicture->Data[1] + SrcPicture->Linesize[1], SrcPicture->Linesize[1] * 2, Dst->GetRowSize(PLANAR_U), Dst->GetHeight(PLANAR_U) / 2);
+			Env->BitBlt(Dst->GetWritePtr(PLANAR_V) + Dst->GetPitch(PLANAR_V), Dst->GetPitch(PLANAR_V) * 2, SrcPicture->Data[2] + SrcPicture->Linesize[2], SrcPicture->Linesize[2] * 2, Dst->GetRowSize(PLANAR_V), Dst->GetHeight(PLANAR_V) / 2);
 		}
 	} else if (VI.IsYUY2()) {
 		if (Field)
-			Env->BitBlt(Dst->GetWritePtr(), Dst->GetPitch() * 2, SrcPicture->Data[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2); 
+			Env->BitBlt(Dst->GetWritePtr(), Dst->GetPitch() * 2, SrcPicture->Data[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2);
 		else
-			Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch(), Dst->GetPitch() * 2, SrcPicture->Data[0] + SrcPicture->Linesize[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2); 
+			Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch(), Dst->GetPitch() * 2, SrcPicture->Data[0] + SrcPicture->Linesize[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2);
 	} else { // RGB
 		if (Field)
-			Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 1), -Dst->GetPitch() * 2, SrcPicture->Data[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2); 
+			Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 1), -Dst->GetPitch() * 2, SrcPicture->Data[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2);
 		else
-			Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 2), -Dst->GetPitch() * 2, SrcPicture->Data[0] + SrcPicture->Linesize[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2); 
+			Env->BitBlt(Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 2), -Dst->GetPitch() * 2, SrcPicture->Data[0] + SrcPicture->Linesize[0], SrcPicture->Linesize[0] * 2, Dst->GetRowSize(), Dst->GetHeight() / 2);
 	}
 }
 
@@ -328,7 +333,7 @@ PVideoFrame AvisynthVideoSource::GetFrame(int n, IScriptEnvironment *Env) {
 			Frame = FFMS_GetFrame(V, FFMAX(FieldList[n].Top, FieldList[n].Bottom), &E);
 			if (Frame == NULL)
 				Env->ThrowError("FFVideoSource: %s", E.Buffer);
-			OutputField(Frame, Dst, !FirstField, Env);			
+			OutputField(Frame, Dst, !FirstField, Env);
 		}
 	} else {
 		const FFMS_Frame *Frame;
@@ -340,13 +345,13 @@ PVideoFrame AvisynthVideoSource::GetFrame(int n, IScriptEnvironment *Env) {
 			Frame = FFMS_GetFrame(V, n, &E);
 			FFMS_Track *T = FFMS_GetTrackFromVideo(V);
 			const FFMS_TrackTimeBase *TB = FFMS_GetTimeBase(T);
-			Env->SetVar("FFVFR_TIME", static_cast<int>(FFMS_GetFrameInfo(T, n)->PTS * static_cast<double>(TB->Num) / TB->Den));
+			Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFVFR_TIME"), static_cast<int>(FFMS_GetFrameInfo(T, n)->PTS * static_cast<double>(TB->Num) / TB->Den));
 		}
 
 		if (Frame == NULL)
 			Env->ThrowError("FFVideoSource: %s", E.Buffer);
 
-		Env->SetVar("FFPICT_TYPE", static_cast<int>(Frame->PictType));
+		Env->SetVar(Env->Sprintf("%s%s", this->VarPrefix, "FFPICT_TYPE"), static_cast<int>(Frame->PictType));
 		OutputFrame(Frame, Dst, Env);
 	}
 
@@ -358,16 +363,15 @@ bool AvisynthVideoSource::GetParity(int n) {
 }
 
 AvisynthAudioSource::AvisynthAudioSource(const char *SourceFile, int Track, FFMS_Index *Index,
-										 int AdjustDelay, IScriptEnvironment* Env) {
+										 int AdjustDelay, const char *VarPrefix, IScriptEnvironment* Env) {
 	memset(&VI, 0, sizeof(VI));
-	SampleOffset = 0;
 
 	char ErrorMsg[1024];
 	FFMS_ErrorInfo E;
 	E.Buffer = ErrorMsg;
 	E.BufferSize = sizeof(ErrorMsg);
 
-	A = FFMS_CreateAudioSource(SourceFile, Track, Index, &E);
+	A = FFMS_CreateAudioSource(SourceFile, Track, Index, AdjustDelay, &E);
 	if (!A)
 		Env->ThrowError("FFAudioSource: %s", E.Buffer);
 
@@ -377,6 +381,11 @@ AvisynthAudioSource::AvisynthAudioSource(const char *SourceFile, int Track, FFMS
 	VI.num_audio_samples = AP->NumSamples;
 	VI.audio_samples_per_second = AP->SampleRate;
 
+	// casting to int should be safe; none of the channel constants are greater than INT_MAX
+	Env->SetVar(Env->Sprintf("%s%s", VarPrefix, "FFCHANNEL_LAYOUT"), static_cast<int>(AP->ChannelLayout));
+
+	Env->SetGlobalVar("FFVAR_PREFIX", VarPrefix);
+
 	switch (AP->SampleFormat) {
 		case FFMS_FMT_U8: VI.sample_type = SAMPLE_INT8; break;
 		case FFMS_FMT_S16: VI.sample_type = SAMPLE_INT16; break;
@@ -384,23 +393,6 @@ AvisynthAudioSource::AvisynthAudioSource(const char *SourceFile, int Track, FFMS
 		case FFMS_FMT_FLT: VI.sample_type = SAMPLE_FLOAT; break;
 		default: Env->ThrowError("FFAudioSource: Bad audio format");
 	}
-
-	if (AdjustDelay >= -2) {
-		double AdjustRelative = 0;
-		if (AdjustDelay >= -1) {
-			int TrackNum = AdjustDelay;
-			if (AdjustDelay == -1)
-				TrackNum = FFMS_GetFirstTrackOfType(Index, FFMS_TYPE_VIDEO, &E);
-
-			if (TrackNum >= 0) {
-				FFMS_Track *VTrack = FFMS_GetTrackFromIndex(Index, TrackNum);
-				AdjustRelative = (FFMS_GetFrameInfo(VTrack, 0)->PTS * FFMS_GetTimeBase(VTrack)->Num) / (double)FFMS_GetTimeBase(VTrack)->Den / 1000;
-			}
-		}
-
-		SampleOffset = static_cast<int64_t>((AdjustRelative - AP->FirstTime) * VI.audio_samples_per_second);
-		VI.num_audio_samples += SampleOffset;
-	}
 }
 
 AvisynthAudioSource::~AvisynthAudioSource() {
@@ -413,14 +405,6 @@ void AvisynthAudioSource::GetAudio(void* Buf, __int64 Start, __int64 Count, IScr
 	E.Buffer = ErrorMsg;
 	E.BufferSize = sizeof(ErrorMsg);
 
-	uint8_t *Buf2 = static_cast<uint8_t *>(Buf);
-	__int64 AdjustedStart = Start + SampleOffset;
-	if (AdjustedStart < 0) {
-		size_t Bytes = static_cast<size_t>(VI.BytesFromAudioSamples(-AdjustedStart));
-		memset(Buf, 0, Bytes);
-		Buf2 += Bytes;
-	}
-
-	if (FFMS_GetAudio(A, Buf2, FFMAX(AdjustedStart, 0), Count + FFMIN(AdjustedStart, 0), &E))
+	if (FFMS_GetAudio(A, Buf, Start, Count, &E))
 		Env->ThrowError("FFAudioSource: %s", E.Buffer);
 }
