@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2009 Fredrik Mellbin
+//  Copyright (c) 2007-2011 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -20,14 +20,29 @@
 
 #include "ffswscale.h"
 #include "avsutils.h"
-#include "../core/utils.h"
+#include "../core/videoutils.h"
+
+static int64_t AvisynthToSWSCPUFlags(long AvisynthFlags) {
+	int64_t Flags = 0;
+#ifdef SWS_CPU_CAPS_MMX
+	if (AvisynthFlags & CPUF_MMX)
+		Flags |= SWS_CPU_CAPS_MMX;
+	if (AvisynthFlags & CPUF_INTEGER_SSE)
+		Flags |= SWS_CPU_CAPS_MMX2;
+	if (AvisynthFlags & CPUF_3DNOW_EXT)
+		Flags |= SWS_CPU_CAPS_3DNOW;
+	if (AvisynthFlags & CPUF_SSE2)
+		Flags |= SWS_CPU_CAPS_SSE2;
+#endif
+	return Flags;
+}
 
 SWScale::SWScale(PClip Child, int ResizeToWidth, int ResizeToHeight, const char *ResizerName, const char *ConvertToFormatName, IScriptEnvironment *Env) : GenericVideoFilter(Child) {
 	Context = NULL;
 	OrigWidth = vi.width;
 	OrigHeight = vi.height;
 	FlipOutput = vi.IsYUV();
-	
+
 	PixelFormat ConvertFromFormat = PIX_FMT_NONE;
 	if (vi.IsYV12())
 		ConvertFromFormat = PIX_FMT_YUV420P;
@@ -71,8 +86,10 @@ SWScale::SWScale(PClip Child, int ResizeToWidth, int ResizeToHeight, const char
 	if ((ConvertToFormat == PIX_FMT_YUV420P || ConvertToFormat == PIX_FMT_YUYV422) && vi.width & 1)
 		Env->ThrowError("SWScale: mod 2 output width required");
 
-	Context = GetSwsContext(OrigWidth, OrigHeight, ConvertFromFormat, vi.width, vi.height, ConvertToFormat, 
-		AvisynthToFFCPUFlags(Env->GetCPUFlags(), false) | Resizer);
+	Context = GetSwsContext(
+		OrigWidth, OrigHeight, ConvertFromFormat, GetAssumedColorSpace(OrigWidth, OrigHeight), AVCOL_RANGE_UNSPECIFIED,
+		vi.width, vi.height, ConvertToFormat, GetAssumedColorSpace(OrigWidth, OrigHeight), AVCOL_RANGE_UNSPECIFIED,
+		AvisynthToSWSCPUFlags(Env->GetCPUFlags()) | Resizer);
 	if (Context == NULL)
 		Env->ThrowError("SWScale: Context creation failed");
 }
@@ -86,7 +103,7 @@ PVideoFrame SWScale::GetFrame(int n, IScriptEnvironment *Env) {
 	PVideoFrame Src = child->GetFrame(n, Env);
 	PVideoFrame Dst = Env->NewVideoFrame(vi);
 
-	FFMS_SWS_CONST_PARAM uint8_t *SrcData[3] = {(uint8_t *)Src->GetReadPtr(PLANAR_Y), (uint8_t *)Src->GetReadPtr(PLANAR_U), (uint8_t *)Src->GetReadPtr(PLANAR_V)};
+	const uint8_t *SrcData[3] = {(uint8_t *)Src->GetReadPtr(PLANAR_Y), (uint8_t *)Src->GetReadPtr(PLANAR_U), (uint8_t *)Src->GetReadPtr(PLANAR_V)};
 	int SrcStride[3] = {Src->GetPitch(PLANAR_Y), Src->GetPitch(PLANAR_U), Src->GetPitch(PLANAR_V)};
 
 	if (FlipOutput) {
