@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2009 Fredrik Mellbin
+//  Copyright (c) 2007-2011 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -19,8 +19,9 @@
 //  THE SOFTWARE.
 
 #ifndef INDEXING_H
-#define	INDEXING_H
+#define INDEXING_H
 
+#include <map>
 #include <memory>
 #include "utils.h"
 #include "wave64writer.h"
@@ -37,13 +38,14 @@
 #	include "guids.h"
 #endif
 
+namespace { struct zipped_file; }
 
 class SharedVideoContext {
-private:
 	bool FreeCodecContext;
 public:
 	AVCodecContext *CodecContext;
 	AVCodecParserContext *Parser;
+	AVBitStreamFilterContext *BitStreamFilter;
 	TrackCompressionContext *TCC;
 
 	SharedVideoContext(bool FreeCodecContext);
@@ -51,7 +53,6 @@ public:
 };
 
 class SharedAudioContext {
-private:
 	bool FreeCodecContext;
 public:
 	AVCodecContext *CodecContext;
@@ -63,43 +64,84 @@ public:
 	~SharedAudioContext();
 };
 
-struct TFrameInfo {
+struct TFrameInfo : public FFMS_FrameInfo {
 public:
-	FFMS_FRAMEINFO_COMMON
 	int64_t SampleStart;
 	unsigned int SampleCount;
 	int64_t FilePos;
 	unsigned int FrameSize;
 	size_t OriginalPos;
+	int FrameType;
 
-	TFrameInfo();
-	static TFrameInfo VideoFrameInfo(int64_t PTS, int RepeatPict, bool KeyFrame, int64_t FilePos = 0, unsigned int FrameSize = 0);
-	static TFrameInfo AudioFrameInfo(int64_t PTS, int64_t SampleStart, unsigned int SampleCount, bool KeyFrame, int64_t FilePos = 0, unsigned int FrameSize = 0);
-private:
-	TFrameInfo(int64_t PTS, int64_t SampleStart, unsigned int SampleCount, int RepeatPict, bool KeyFrame, int64_t FilePos, unsigned int FrameSize);
+	TFrameInfo() { }
+	TFrameInfo(int64_t PTS, int64_t SampleStart, unsigned int SampleCount, int RepeatPict, bool KeyFrame, int64_t FilePos, unsigned int FrameSize, int FrameType);
 };
 
-class FFMS_Track : public std::vector<TFrameInfo> {
+struct FFMS_Track {
+private:
+	typedef std::vector<TFrameInfo> frame_vec;
+	frame_vec Frames;
+	std::vector<size_t> InvisibleFrames;
+
+	void MaybeReorderFrames();
+	void MaybeHideFrames();
+
 public:
 	FFMS_TrackType TT;
 	FFMS_TrackTimeBase TB;
 	bool UseDTS;
+	bool HasTS;
 
-	int FindClosestVideoKeyFrame(int Frame);
-	int FindClosestAudioKeyFrame(int64_t Sample);
-	int FrameFromPTS(int64_t PTS);
-	int ClosestFrameFromPTS(int64_t PTS);
-	void WriteTimecodes(const char *TimecodeFile);
+	void AddVideoFrame(int64_t PTS, int RepeatPict, bool KeyFrame, int FrameType, int64_t FilePos = 0, unsigned int FrameSize = 0, bool Invisible = false);
+	void AddAudioFrame(int64_t PTS, int64_t SampleStart, int64_t SampleCount, bool KeyFrame, int64_t FilePos = 0, unsigned int FrameSize = 0);
+
+	void SortByPTS();
+
+	int FindClosestVideoKeyFrame(int Frame) const;
+	int FrameFromPTS(int64_t PTS) const;
+	int FrameFromPos(int64_t Pos) const;
+	int ClosestFrameFromPTS(int64_t PTS) const;
+	int RealFrameNumber(int Frame) const;
+	int VisibleFrameCount() const;
+
+	void WriteTimecodes(const char *TimecodeFile) const;
+	void Write(zipped_file *Stream) const;
+
+	typedef frame_vec::allocator_type allocator_type;
+	typedef frame_vec::size_type size_type;
+	typedef frame_vec::difference_type difference_type;
+	typedef frame_vec::const_pointer pointer;
+	typedef frame_vec::const_reference reference;
+	typedef frame_vec::value_type value_type;
+	typedef frame_vec::const_iterator iterator;
+	typedef frame_vec::const_reverse_iterator reverse_iterator;
+
+	void clear() { Frames.clear(); InvisibleFrames.clear(); }
+
+	bool empty() const { return Frames.empty(); }
+	size_type size() const { return Frames.size(); }
+	reference operator[](size_type pos) const { return Frames[pos]; }
+	reference front() const { return Frames.front(); }
+	reference back() const { return Frames.back(); }
+	iterator begin() const { return Frames.begin(); }
+	iterator end() const { return Frames.end(); }
 
 	FFMS_Track();
-	FFMS_Track(int64_t Num, int64_t Den, FFMS_TrackType TT, bool UseDTS = false);
+	FFMS_Track(zipped_file &Stream);
+	FFMS_Track(int64_t Num, int64_t Den, FFMS_TrackType TT, bool UseDTS = false, bool HasTS = true);
 };
 
-class FFMS_Index : public std::vector<FFMS_Track> {
+struct FFMS_Index : public std::vector<FFMS_Track> {
+private:
+	int RefCount;
 public:
 	static void CalculateFileSignature(const char *Filename, int64_t *Filesize, uint8_t Digest[20]);
 
+	int AddRef();
+	int Release();
+
 	int Decoder;
+	int ErrorHandling;
 	int64_t Filesize;
 	uint8_t Digest[20];
 
@@ -112,7 +154,8 @@ public:
 	FFMS_Index(int64_t Filesize, uint8_t Digest[20]);
 };
 
-class FFMS_Indexer {
+struct FFMS_Indexer {
+	std::map<int, FFMS_AudioProperties> LastAudioProperties;
 protected:
 	int IndexMask;
 	int DumpMask;
@@ -121,15 +164,19 @@ protected:
 	void *ICPrivate;
 	TAudioNameCallback ANC;
 	void *ANCPrivate;
-	const char *SourceFile;
-	AlignedBuffer<int16_t> DecodingBuffer;
+	std::string SourceFile;
+	ScopedFrame DecodeFrame;
 
 	int64_t Filesize;
 	uint8_t Digest[20];
 
-	void WriteAudio(SharedAudioContext &AudioContext, FFMS_Index *Index, int Track, int DBSize);
+	void WriteAudio(SharedAudioContext &AudioContext, FFMS_Index *Index, int Track);
+	void CheckAudioProperties(int Track, AVCodecContext *Context);
+	int64_t IndexAudioPacket(int Track, AVPacket *Packet, SharedAudioContext &Context, FFMS_Index &TrackIndices);
+	void ParseVideoPacket(SharedVideoContext &VideoContext, AVPacket &pkt, int *RepeatPict, int *FrameType, bool *Invisible);
+
 public:
-	static FFMS_Indexer *CreateIndexer(const char *Filename);
+	static FFMS_Indexer *CreateIndexer(const char *Filename, FFMS_Sources Demuxer = FFMS_SOURCE_DEFAULT);
 	FFMS_Indexer(const char *Filename);
 	virtual ~FFMS_Indexer();
 	void SetIndexMask(int IndexMask);
@@ -141,11 +188,13 @@ public:
 	virtual int GetNumberOfTracks() = 0;
 	virtual FFMS_TrackType GetTrackType(int Track) = 0;
 	virtual const char *GetTrackCodec(int Track) = 0;
+	virtual FFMS_Sources GetSourceType() = 0;
+	virtual const char *GetFormatName() = 0;
 };
 
 class FFLAVFIndexer : public FFMS_Indexer {
-private:
 	AVFormatContext *FormatContext;
+	void ReadTS(const AVPacket &Packet, int64_t &TS, bool &UseDTS);
 public:
 	FFLAVFIndexer(const char *Filename, AVFormatContext *FormatContext);
 	~FFLAVFIndexer();
@@ -153,6 +202,8 @@ public:
 	int GetNumberOfTracks();
 	FFMS_TrackType GetTrackType(int Track);
 	const char *GetTrackCodec(int Track);
+	const char *GetFormatName();
+	FFMS_Sources GetSourceType();
 };
 
 class FFMatroskaIndexer : public FFMS_Indexer {
@@ -167,6 +218,8 @@ public:
 	int GetNumberOfTracks();
 	FFMS_TrackType GetTrackType(int Track);
 	const char *GetTrackCodec(int Track);
+	const char *GetFormatName();
+	FFMS_Sources GetSourceType();
 };
 
 #ifdef HAALISOURCE
@@ -177,17 +230,17 @@ private:
 	CComPtr<IMMContainer> pMMC;
 	int NumTracks;
 	FFMS_TrackType TrackType[32];
-	AVCodec *Codec[32];
-	std::vector<uint8_t> CodecPrivate[32];
-	int CodecPrivateSize[32];
 	CComQIPtr<IPropertyBag> PropertyBags[32];
 	int64_t Duration;
+	AVFormatContext *FormatContext;
 public:
-	FFHaaliIndexer(const char *Filename, enum FFMS_Sources SourceMode);
+	FFHaaliIndexer(const char *Filename, FFMS_Sources SourceMode);
 	FFMS_Index *DoIndexing();
 	int GetNumberOfTracks();
 	FFMS_TrackType GetTrackType(int Track);
 	const char *GetTrackCodec(int Track);
+	const char *GetFormatName();
+	FFMS_Sources GetSourceType();
 };
 
 #endif // HAALISOURCE
