@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2009 Fredrik Mellbin
+//  Copyright (c) 2007-2011 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,16 @@
 
 #include <string.h>
 #include <errno.h>
+#include <algorithm>
+
+// avcodec.h includes audioconvert.h, but we need to include audioconvert.h
+// ourselves later
+#define FF_API_OLD_AUDIOCONVERT 0
+#define AVUTIL_AUDIOCONVERT_H
+
 #include "utils.h"
+
+#include "codectype.h"
 #include "indexing.h"
 
 #ifdef _WIN32
@@ -29,38 +38,16 @@
 #	include <io.h>
 #	include <fcntl.h>
 extern "C" {
-#	include "libavutil/avstring.h"
+#	include <libavutil/avstring.h>
 }
 #endif // _WIN32
 
-
-// Export the array but not its data type... fun...
-typedef struct CodecTags{
-    char str[20];
-    enum CodecID id;
-} CodecTags;
-
-extern "C" {
-extern const AVCodecTag ff_codec_bmp_tags[];
-extern const CodecTags ff_mkv_codec_tags[];
-extern const AVCodecTag ff_codec_movvideo_tags[];
-extern const AVCodecTag ff_codec_wav_tags[];
-
-/* if you have this, we'll assume you have a new enough libavutil too */
-#if LIBSWSCALE_VERSION_INT >= AV_VERSION_INT(0, 12, 0)
-#	include <libavutil/opt.h>
-#endif
-}
-
-extern int CPUFeatures;
 extern bool GlobalUseUTF8Paths;
 
-
-
-FFMS_Exception::FFMS_Exception(int ErrorType, int SubType, const char *Message) : _ErrorType(ErrorType), _SubType(SubType), _Message(Message) {
+FFMS_Exception::FFMS_Exception(int ErrorType, int SubType, const char *Message) : _Message(Message), _ErrorType(ErrorType), _SubType(SubType) {
 }
 
-FFMS_Exception::FFMS_Exception(int ErrorType, int SubType, const std::string &Message) : _ErrorType(ErrorType), _SubType(SubType), _Message(Message) {
+FFMS_Exception::FFMS_Exception(int ErrorType, int SubType, const std::string &Message) : _Message(Message), _ErrorType(ErrorType), _SubType(SubType) {
 }
 
 FFMS_Exception::~FFMS_Exception() throw () {
@@ -112,91 +99,6 @@ TrackCompressionContext::~TrackCompressionContext() {
 		cs_Destroy(CS);
 }
 
-int64_t GetSWSCPUFlags() {
-	int64_t Flags = 0;
-
-	if (CPUFeatures & FFMS_CPU_CAPS_MMX)
-		Flags |= SWS_CPU_CAPS_MMX;
-	if (CPUFeatures & FFMS_CPU_CAPS_MMX2)
-		Flags |= SWS_CPU_CAPS_MMX2;
-	if (CPUFeatures & FFMS_CPU_CAPS_3DNOW)
-		Flags |= SWS_CPU_CAPS_3DNOW;
-	if (CPUFeatures & FFMS_CPU_CAPS_ALTIVEC)
-		Flags |= SWS_CPU_CAPS_ALTIVEC;
-	if (CPUFeatures & FFMS_CPU_CAPS_BFIN)
-		Flags |= SWS_CPU_CAPS_BFIN;
-#ifdef SWS_CPU_CAPS_SSE2
-	if (CPUFeatures & FFMS_CPU_CAPS_SSE2)
-		Flags |= SWS_CPU_CAPS_SSE2;
-#endif
-
-	return Flags;
-}
-
-static int handle_jpeg(PixelFormat *format)
-{
-	switch (*format) {
-	case PIX_FMT_YUVJ420P: *format = PIX_FMT_YUV420P; return 1;
-	case PIX_FMT_YUVJ422P: *format = PIX_FMT_YUV422P; return 1;
-	case PIX_FMT_YUVJ444P: *format = PIX_FMT_YUV444P; return 1;
-	case PIX_FMT_YUVJ440P: *format = PIX_FMT_YUV440P; return 1;
-	default:                                          return 0;
-	}
-}
-SwsContext *GetSwsContext(int SrcW, int SrcH, PixelFormat SrcFormat, int DstW, int DstH, PixelFormat DstFormat, int64_t Flags, int ColorSpace) {
-#if LIBSWSCALE_VERSION_INT < AV_VERSION_INT(0, 12, 0)
-	return sws_getContext(SrcW, SrcH, SrcFormat, DstW, DstH, DstFormat, Flags, 0, 0, 0);
-#else
-	SwsContext *Context = sws_alloc_context();
-	if (!Context) return 0;
-
-	if (ColorSpace == -1)
-		ColorSpace = (SrcW > 1024 || SrcH >= 600) ? SWS_CS_ITU709 : SWS_CS_DEFAULT;
-
-	int SrcRange = handle_jpeg(&SrcFormat);
-	int DstRange = handle_jpeg(&DstFormat);
-
-	av_set_int(Context, "sws_flags", Flags);
-	av_set_int(Context, "srcw",       SrcW);
-	av_set_int(Context, "srch",       SrcH);
-	av_set_int(Context, "dstw",       DstW);
-	av_set_int(Context, "dsth",       DstH);
-	av_set_int(Context, "src_range",  SrcRange);
-	av_set_int(Context, "dst_range",  DstRange);
-	av_set_int(Context, "src_format", SrcFormat);
-	av_set_int(Context, "dst_format", DstFormat);
-
-	sws_setColorspaceDetails(Context, sws_getCoefficients(ColorSpace), SrcRange, sws_getCoefficients(ColorSpace), DstRange, 0, 1<<16, 1<<16);
-
-	if(sws_init_context(Context, 0, 0) < 0){
-		sws_freeContext(Context);
-		return 0;
-	}
-
-	return Context;
-#endif
-
-}
-
-
-int GetPPCPUFlags() {
-	int Flags = 0;
-
-#ifdef FFMS_USE_POSTPROC
-// not exactly a pretty solution but it'll never get called anyway
-	if (CPUFeatures & FFMS_CPU_CAPS_MMX)
-		Flags |= PP_CPU_CAPS_MMX;
-	if (CPUFeatures & FFMS_CPU_CAPS_MMX2)
-		Flags |= PP_CPU_CAPS_MMX2;
-	if (CPUFeatures & FFMS_CPU_CAPS_3DNOW)
-		Flags |= PP_CPU_CAPS_3DNOW;
-	if (CPUFeatures & FFMS_CPU_CAPS_ALTIVEC)
-		Flags |= PP_CPU_CAPS_ALTIVEC;
-#endif // FFMS_USE_POSTPROC
-
-	return Flags;
-}
-
 void ClearErrorInfo(FFMS_ErrorInfo *ErrorInfo) {
 	if (ErrorInfo) {
 		ErrorInfo->ErrorType = FFMS_ERROR_SUCCESS;
@@ -207,30 +109,10 @@ void ClearErrorInfo(FFMS_ErrorInfo *ErrorInfo) {
 	}
 }
 
-FFMS_TrackType HaaliTrackTypeToFFTrackType(int TT) {
-	switch (TT) {
-		case TT_VIDEO: return FFMS_TYPE_VIDEO; break;
-		case TT_AUDIO: return FFMS_TYPE_AUDIO; break;
-		case TT_SUB: return FFMS_TYPE_SUBTITLE; break;
-		default: return FFMS_TYPE_UNKNOWN;
-	}
-}
-
-const char *GetLAVCSampleFormatName(AVSampleFormat s) {
-	switch (s) {
-		case AV_SAMPLE_FMT_U8:	return "8-bit unsigned integer";
-		case AV_SAMPLE_FMT_S16:	return "16-bit signed integer";
-		case AV_SAMPLE_FMT_S32:	return "32-bit signed integer";
-		case AV_SAMPLE_FMT_FLT:	return "Single-precision floating point";
-		case AV_SAMPLE_FMT_DBL:	return "Double-precision floating point";
-		default:				return "Unknown";
-	}
-}
-
 template<class T> static void safe_aligned_reallocz(T *&ptr, size_t old_size, size_t new_size) {
 	void *newalloc = av_mallocz(new_size);
 	if (newalloc) {
-		memcpy(newalloc, ptr, min(old_size, new_size));
+		memcpy(newalloc, ptr, FFMIN(old_size, new_size));
 	}
 	av_free(ptr);
 	ptr = static_cast<T*>(newalloc);
@@ -238,7 +120,7 @@ template<class T> static void safe_aligned_reallocz(T *&ptr, size_t old_size, si
 
 void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContext *TCC, MatroskaReaderContext &Context) {
 	memset(Context.Buffer, 0, Context.BufferSize); // necessary to avoid lavc hurfing a durf with some mpeg4 video streams
-	if (TCC && TCC->CS) {
+	if (TCC && TCC->CompressionMethod == COMP_ZLIB) {
 		CompressedStream *CS = TCC->CS;
 		unsigned int DecompressedFrameSize = 0;
 
@@ -325,6 +207,8 @@ void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContex
 				"Unknown read error");
 		}
 
+		if (TCC && TCC->CompressionMethod == COMP_PREPEND)
+			FrameSize += TCC->CompressedPrivateDataSize;
 		return;
 	}
 }
@@ -335,20 +219,51 @@ void InitNullPacket(AVPacket &pkt) {
 	pkt.size = 0;
 }
 
-void FillAP(FFMS_AudioProperties &AP, AVCodecContext *CTX, FFMS_Track &Frames) {
-	AP.SampleFormat = static_cast<FFMS_SampleFormat>(CTX->sample_fmt);
-	AP.BitsPerSample = av_get_bits_per_sample_fmt(CTX->sample_fmt);
-	if (CTX->sample_fmt == AV_SAMPLE_FMT_S32 && CTX->bits_per_raw_sample)
-		AP.BitsPerSample = CTX->bits_per_raw_sample;
+extern "C" {
+#if VERSION_CHECK(LIBAVUTIL_VERSION_INT, >=, 52, 2, 0, 52, 6, 100)
+#include <libavutil/channel_layout.h>
+#else
+#undef AVUTIL_AUDIOCONVERT_H
 
-	AP.Channels = CTX->channels;;
+// Whether or not av_get_default_channel_layout exists in a given version
+// depends on which branch that version is from, since FFmpeg doesn't
+// understand the concept of version numbers. Work around this by always using
+// our copy, since that's less effort than detecting whether or not it's
+// available.
+#define av_get_default_channel_layout av_get_default_channel_layout_hurr
+#include "libavutil/audioconvert.h"
+#undef av_get_default_channel_layout
+
+static int64_t av_get_default_channel_layout(int nb_channels) {
+	switch(nb_channels) {
+		case 1: return AV_CH_LAYOUT_MONO;
+		case 2: return AV_CH_LAYOUT_STEREO;
+		case 3: return AV_CH_LAYOUT_SURROUND;
+		case 4: return AV_CH_LAYOUT_QUAD;
+		case 5: return AV_CH_LAYOUT_5POINT0;
+		case 6: return AV_CH_LAYOUT_5POINT1;
+		case 7: return AV_CH_LAYOUT_6POINT1;
+		case 8: return AV_CH_LAYOUT_7POINT1;
+		default: return 0;
+	}
+}
+#endif
+}
+
+void FillAP(FFMS_AudioProperties &AP, AVCodecContext *CTX, FFMS_Track &Frames) {
+	AP.SampleFormat = static_cast<FFMS_SampleFormat>(av_get_packed_sample_fmt(CTX->sample_fmt));
+	AP.BitsPerSample = av_get_bytes_per_sample(CTX->sample_fmt) * 8;
+	AP.Channels = CTX->channels;
 	AP.ChannelLayout = CTX->channel_layout;
 	AP.SampleRate = CTX->sample_rate;
-	if (Frames.size() > 0) {
+	if (!Frames.empty()) {
 		AP.NumSamples = (Frames.back()).SampleStart + (Frames.back()).SampleCount;
 		AP.FirstTime = ((Frames.front().PTS * Frames.TB.Num) / (double)Frames.TB.Den) / 1000;
 		AP.LastTime = ((Frames.back().PTS * Frames.TB.Num) / (double)Frames.TB.Den) / 1000;
 	}
+
+	if (AP.ChannelLayout == 0)
+		AP.ChannelLayout = av_get_default_channel_layout(AP.Channels);
 }
 
 #ifdef HAALISOURCE
@@ -376,67 +291,36 @@ void vtCopy(VARIANT& vt,void *dest) {
 
 #endif
 
-CodecID MatroskaToFFCodecID(char *Codec, void *CodecPrivate, unsigned int FourCC, unsigned int BitsPerSample) {
-/* Look up native codecs */
-	for(int i = 0; ff_mkv_codec_tags[i].id != CODEC_ID_NONE; i++){
-		if(!strncmp(ff_mkv_codec_tags[i].str, Codec,
-			strlen(ff_mkv_codec_tags[i].str))) {
-
-				// Uncompressed and exotic format fixup
-				// This list is incomplete
-				CodecID CID = ff_mkv_codec_tags[i].id;
-				switch (CID) {
-					case CODEC_ID_PCM_S16LE:
-						switch (BitsPerSample) {
-							case 8: CID = CODEC_ID_PCM_S8; break;
-							case 16: CID = CODEC_ID_PCM_S16LE; break;
-							case 24: CID = CODEC_ID_PCM_S24LE; break;
-							case 32: CID = CODEC_ID_PCM_S32LE; break;
-						}
-						break;
-					case CODEC_ID_PCM_S16BE:
-						switch (BitsPerSample) {
-							case 8: CID = CODEC_ID_PCM_S8; break;
-							case 16: CID = CODEC_ID_PCM_S16BE; break;
-							case 24: CID = CODEC_ID_PCM_S24BE; break;
-							case 32: CID = CODEC_ID_PCM_S32BE; break;
-						}
-						break;
-					default:
-						break;
-				}
-
-				return CID;
-			}
-	}
-
-/* Video codecs for "avi in mkv" mode */
-	const AVCodecTag *const tags[] = { ff_codec_bmp_tags, 0 };
-
-	if (!strcmp(Codec, "V_MS/VFW/FOURCC")) {
-		FFMS_BITMAPINFOHEADER *b = reinterpret_cast<FFMS_BITMAPINFOHEADER *>(CodecPrivate);
-		return av_codec_get_id(tags, b->biCompression);
-	}
-
-	if (!strcmp(Codec, "V_FOURCC")) {
-		return av_codec_get_id(tags, FourCC);
-	}
-
-// FIXME
-/* Audio codecs for "acm in mkv" mode */
-		//#include "Mmreg.h"
-		//((WAVEFORMATEX *)TI->CodecPrivate)->wFormatTag
-
-/* Fixup for uncompressed video formats */
-
-/* Fixup for uncompressed audio formats */
-
-	return CODEC_ID_NONE;
-}
-
 void InitializeCodecContextFromMatroskaTrackInfo(TrackInfo *TI, AVCodecContext *CodecContext) {
-	CodecContext->extradata = static_cast<uint8_t *>(TI->CodecPrivate);
-	CodecContext->extradata_size = TI->CodecPrivateSize;
+	uint8_t *PrivateDataSrc = static_cast<uint8_t *>(TI->CodecPrivate);
+	size_t PrivateDataSize = TI->CodecPrivateSize;
+	size_t BIHSize = sizeof(FFMS_BITMAPINFOHEADER); // 40 bytes
+	if (!strncmp(TI->CodecID, "V_MS/VFW/FOURCC", 15) && PrivateDataSize >= BIHSize) {
+		// For some reason UTVideo requires CodecContext->codec_tag (i.e. the FourCC) to be set.
+		// Fine, it can't hurt to set it, so let's go find it.
+		// In a V_MS/VFW/FOURCC track, the codecprivate starts with a BITMAPINFOHEADER. If you treat that struct
+		// as an array of uint32_t, the biCompression member (that's the FourCC) can be found at offset 4.
+		// Therefore the following derp.
+		CodecContext->codec_tag = reinterpret_cast<uint32_t *>(PrivateDataSrc)[4];
+
+		// Now skip copying the BITMAPINFOHEADER into the extradata, because lavc doesn't expect it to be there.
+		if (PrivateDataSize <= BIHSize) {
+			PrivateDataSrc = NULL;
+			PrivateDataSize = 0;
+		}
+		else {
+			PrivateDataSrc += BIHSize;
+			PrivateDataSize -= BIHSize;
+		}
+	}
+	// I think you might need to do some special handling for A_MS/ACM extradata too,
+	// but I don't think anyone actually uses that.
+
+	if (PrivateDataSrc && PrivateDataSize > 0) {
+		CodecContext->extradata = static_cast<uint8_t *>(av_mallocz(PrivateDataSize + FF_INPUT_BUFFER_PADDING_SIZE));
+		CodecContext->extradata_size = PrivateDataSize;
+		memcpy(CodecContext->extradata, PrivateDataSrc, PrivateDataSize);
+	}
 
 	if (TI->Type == TT_VIDEO) {
 		CodecContext->coded_width = TI->AV.Video.PixelWidth;
@@ -457,7 +341,7 @@ FFCodecContext InitializeCodecContextFromHaaliInfo(CComQIPtr<IPropertyBag> pBag)
 
 	unsigned int TT = pV.uintVal;
 
-	FFCodecContext CodecContext(avcodec_alloc_context(), DeleteHaaliCodecContext);
+	FFCodecContext CodecContext(avcodec_alloc_context3(NULL), DeleteHaaliCodecContext);
 
 	unsigned int FourCC = 0;
 	if (TT == TT_VIDEO) {
@@ -473,38 +357,18 @@ FFCodecContext InitializeCodecContextFromHaaliInfo(CComQIPtr<IPropertyBag> pBag)
 		if (SUCCEEDED(pBag->Read(L"FOURCC", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
 			FourCC = pV.uintVal;
 
-		FFMS_BITMAPINFOHEADER bih;
-		memset(&bih, 0, sizeof bih);
-		if (FourCC) {
-			// Reconstruct the missing codec private part for VC1
-			bih.biSize = sizeof bih;
-			bih.biCompression = FourCC;
-			bih.biBitCount = 24;
-			bih.biPlanes = 1;
-			bih.biHeight = CodecContext->coded_height;
-		}
-
 		pV.Clear();
 		if (SUCCEEDED(pBag->Read(L"CodecPrivate", &pV, NULL))) {
-			bih.biSize += vtSize(pV);
-			CodecContext->extradata = static_cast<uint8_t*>(av_malloc(bih.biSize));
-			// prepend BITMAPINFOHEADER if there's anything interesting in it (i.e. we're decoding VC1)
-			if (FourCC)
-				memcpy(CodecContext->extradata, &bih, sizeof bih); 
-			vtCopy(pV, CodecContext->extradata + (FourCC ? sizeof bih : 0));
+			CodecContext->extradata_size = vtSize(pV);
+			CodecContext->extradata = static_cast<uint8_t*>(av_mallocz(CodecContext->extradata_size + FF_INPUT_BUFFER_PADDING_SIZE));
+			vtCopy(pV, CodecContext->extradata);
 		}
-		// use the BITMAPINFOHEADER only. not sure if this is correct or if it's ever going to be used...
-		else {
-			CodecContext->extradata = static_cast<uint8_t*>(av_malloc(bih.biSize));
-			memcpy(CodecContext->extradata, &bih, sizeof bih);
-		}
-		CodecContext->extradata_size = bih.biSize;
 	}
 	else if (TT == TT_AUDIO) {
 		pV.Clear();
 		if (SUCCEEDED(pBag->Read(L"CodecPrivate", &pV, NULL))) {
 			CodecContext->extradata_size = vtSize(pV);
-			CodecContext->extradata = static_cast<uint8_t*>(av_malloc(CodecContext->extradata_size));
+			CodecContext->extradata = static_cast<uint8_t*>(av_mallocz(CodecContext->extradata_size + FF_INPUT_BUFFER_PADDING_SIZE));
 			vtCopy(pV, CodecContext->extradata);
 		}
 
@@ -539,47 +403,40 @@ FFCodecContext InitializeCodecContextFromHaaliInfo(CComQIPtr<IPropertyBag> pBag)
 // work correctly on Win32.
 // It's a really ugly hack, and I blame Microsoft for it.
 #ifdef _WIN32
-static wchar_t *dup_char_to_wchar(const char *s, unsigned int cp) {
-	wchar_t *w;
-	int l;
-	if (!(l = MultiByteToWideChar(cp, MB_ERR_INVALID_CHARS, s, -1, NULL, 0)))
-		return NULL;
-	if (!(w = (wchar_t *)malloc(l * sizeof(wchar_t))))
-		return NULL;
-	if (MultiByteToWideChar(cp, MB_ERR_INVALID_CHARS, s, -1 , w, l) <= 0) {
-		free(w);
-		w = NULL;
-	}
-	return w;
+static std::wstring char_to_wstring(const char *s, unsigned int cp) {
+	std::wstring ret;
+	std::vector<wchar_t> tmp;
+	int len;
+	if (!(len = MultiByteToWideChar(cp, MB_ERR_INVALID_CHARS, s, -1, NULL, 0)))
+		return ret;
+
+	tmp.resize(len);
+	if (MultiByteToWideChar(cp, MB_ERR_INVALID_CHARS, s, -1 , &tmp[0], len) <= 0)
+		return ret;
+
+	ret.assign(&tmp[0]);
+	return ret;
+}
+
+static std::wstring widen_path(const char *s) {
+	return char_to_wstring(s, GlobalUseUTF8Paths ? CP_UTF8 : CP_ACP);
 }
 #endif
 
 FILE *ffms_fopen(const char *filename, const char *mode) {
 #ifdef _WIN32
-	unsigned int codepage;
-	if (GlobalUseUTF8Paths)
-		codepage = CP_UTF8;
+	std::wstring filename_wide = widen_path(filename);
+	std::wstring mode_wide     = widen_path(mode);
+	if (filename_wide.size() && mode_wide.size())
+		return _wfopen(filename_wide.c_str(), mode_wide.c_str());
 	else
-		codepage = CP_ACP;
-
-	FILE *ret;
-	wchar_t *filename_wide	= dup_char_to_wchar(filename, codepage);
-	wchar_t *mode_wide		= dup_char_to_wchar(mode, codepage);
-	if (filename_wide && mode_wide)
-		ret = _wfopen(filename_wide, mode_wide);
-	else
-		ret = fopen(filename, mode);
-
-	free(filename_wide);
-	free(mode_wide);
-
-	return ret;
+		return fopen(filename, mode);
 #else
 	return fopen(filename, mode);
 #endif /* _WIN32 */
 }
 
-size_t ffms_mbstowcs (wchar_t *wcstr, const char *mbstr, size_t max) {
+size_t ffms_mbstowcs(wchar_t *wcstr, const char *mbstr, size_t max) {
 #ifdef _WIN32
 	// this is only called by HaaliOpenFile anyway, so I think this is safe
 	return static_cast<size_t>(MultiByteToWideChar((GlobalUseUTF8Paths ? CP_UTF8 : CP_ACP), MB_ERR_INVALID_CHARS, mbstr, -1, wcstr, max));
@@ -588,88 +445,55 @@ size_t ffms_mbstowcs (wchar_t *wcstr, const char *mbstr, size_t max) {
 #endif
 }
 
+#ifdef __MINGW32__
+static int open_flags(std::ios::openmode mode) {
+	int flags = 0;
+	if ((mode & std::ios::in) && (mode & std::ios::out))
+		flags = _O_CREAT | _O_TRUNC | _O_RDWR;
+	if (mode & std::ios::in)
+		flags = _O_RDONLY;
+	else if (mode & std::ios::out)
+		flags = _O_CREAT | _O_TRUNC | _O_WRONLY;
+
+#ifdef _O_BINARY
+	flags |= _O_BINARY;
+#endif
+	return flags;
+}
+#endif
 
 // ffms_fstream stuff
-void ffms_fstream::open(const char *filename, std::ios_base::openmode mode) {
-	// Unlike MSVC, mingw's iostream library doesn't have an fstream overload
-	// that takes a wchar_t* filename, which means you can't open unicode
-	// filenames with it on Windows. gg.
-#if defined(_WIN32) && !defined(__MINGW32__)
-	unsigned int codepage = GlobalUseUTF8Paths ? CP_UTF8 : CP_ACP;
-
-	wchar_t *filename_wide = dup_char_to_wchar(filename, codepage);
-	if (filename_wide)
-		std::fstream::open(filename_wide, mode);
-	else
-		std::fstream::open(filename, mode);
-
-	free(filename_wide);
-#else // defined(_WIN32) && !defined(__MINGW32__)
-	std::fstream::open(filename, mode);
-#endif // defined(_WIN32) && !defined(__MINGW32__)
-}
-
-ffms_fstream::ffms_fstream(const char *filename, std::ios_base::openmode mode) {
-	open(filename, mode);
-}
-
-
-#ifdef _WIN32
-int ffms_wchar_open(const char *fname, int oflags, int pmode) {
-    wchar_t *wfname = dup_char_to_wchar(fname, CP_UTF8);
-    if (wfname) {
-        int ret = _wopen(wfname, oflags, pmode);
-        free(wfname);
-        return ret;
-    }
-    return -1;
-}
-
-static int ffms_lavf_file_open(URLContext *h, const char *filename, int flags) {
-    int access;
-    int fd;
-
-    av_strstart(filename, "file:", &filename);
-
-    if (flags & URL_RDWR) {
-        access = _O_CREAT | _O_TRUNC | _O_RDWR;
-    } else if (flags & URL_WRONLY) {
-        access = _O_CREAT | _O_TRUNC | _O_WRONLY;
-    } else {
-        access = _O_RDONLY;
-    }
-#ifdef _O_BINARY
-    access |= _O_BINARY;
+ffms_fstream::ffms_fstream(const char *filename, std::ios_base::openmode mode)
+#ifdef __MINGW32__
+: filebuf(_wopen(widen_path(filename).c_str(), open_flags(mode), 0666), mode)
 #endif
-    fd = ffms_wchar_open(filename, access, 0666);
-    if (fd == -1)
-        return AVERROR(ENOENT);
-    h->priv_data = (void *) (intptr_t) fd;
-    return 0;
-}
+{
+#if defined(_WIN32)
 
-// Hijack lavf's file protocol handler's open function and use our own instead.
-// Hack by nielsm.
-void ffms_patch_lavf_file_open() {
-	URLProtocol *proto = av_protocol_next(NULL);
-	while (proto != NULL) {
-		if (strcmp("file", proto->name) == 0) {
-			break;
-		}
-		proto = proto->next;
-	}
-	if (proto != NULL) {
-		proto->url_open = &ffms_lavf_file_open;
-	}
-}
+#ifndef __MINGW32__
+	std::wstring filename_wide = widen_path(filename);
+	if (filename_wide.size())
+		open(filename_wide.c_str(), mode);
+	else
+		open(filename, mode);
+#else
+	// Unlike MSVC, mingw's iostream library doesn't have an fstream overload
+	// that takes a wchar_t* filename, so instead we use gcc's nonstandard
+	// fd wrapper
+	std::iostream::rdbuf(&filebuf);
+#endif //__MINGW32__
+
+#else // _WIN32
+	open(filename, mode);
 #endif // _WIN32
+}
 
 // End of filename hackery.
 
 
 #ifdef HAALISOURCE
 
-CComPtr<IMMContainer> HaaliOpenFile(const char *SourceFile, enum FFMS_Sources SourceMode) {
+CComPtr<IMMContainer> HaaliOpenFile(const char *SourceFile, FFMS_Sources SourceMode) {
 	CComPtr<IMMContainer> pMMC;
 
 	CLSID clsid = HAALI_MPEG_PARSER;
@@ -712,50 +536,28 @@ CComPtr<IMMContainer> HaaliOpenFile(const char *SourceFile, enum FFMS_Sources So
 #endif
 
 void LAVFOpenFile(const char *SourceFile, AVFormatContext *&FormatContext) {
-	if (av_open_input_file(&FormatContext, SourceFile, NULL, 0, NULL) != 0)
+	if (avformat_open_input(&FormatContext, SourceFile, NULL, NULL) != 0)
 		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
 			std::string("Couldn't open '") + SourceFile + "'");
 
-	if (av_find_stream_info(FormatContext) < 0) {
-		av_close_input_file(FormatContext);
+	if (avformat_find_stream_info(FormatContext,NULL) < 0) {
+		avformat_close_input(&FormatContext);
 		FormatContext = NULL;
 		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
 			"Couldn't find stream information");
 	}
 }
 
-
-// attempt to correct framerate to the proper NTSC fraction, if applicable
-// code stolen from Perian
-void CorrectNTSCRationalFramerate(int *Num, int *Den) {
-	AVRational TempFPS;
-	TempFPS.den = *Num; // not a typo
-	TempFPS.num = *Den; // still not a typo
-
-	av_reduce(&TempFPS.num, &TempFPS.den, TempFPS.num, TempFPS.den, INT_MAX);
-
-	if (TempFPS.num == 1) {
-		*Num = TempFPS.den;
-		*Den = TempFPS.num;
-	}
+void FlushBuffers(AVCodecContext *CodecContext) {
+	if (CodecContext->codec->flush)
+		avcodec_flush_buffers(CodecContext);
 	else {
-		double FTimebase = av_q2d(TempFPS);
-		double NearestNTSC = floor(FTimebase * 1001.0 + 0.5) / 1001.0;
-		const double SmallInterval = 1.0/120.0;
-
-		if (fabs(FTimebase - NearestNTSC) < SmallInterval) {
-			*Num = int((1001.0 / FTimebase) + 0.5);
-			*Den = 1001;
-		}
-	}
-}
-
-// correct the timebase if it is invalid
-void CorrectTimebase(FFMS_VideoProperties *VP, FFMS_TrackTimeBase *TTimebase) {
-	double Timebase = (double)TTimebase->Num / TTimebase->Den;
-	double FPS = (double)VP->FPSNumerator / VP->FPSDenominator;
-	if ((1000/Timebase) / FPS < 1) {
-		TTimebase->Den = VP->FPSNumerator;
-		TTimebase->Num = (int64_t)VP->FPSDenominator * 1000;
+		// If the codec doesn't have flush(), it might not need it... or it
+		// might need it and just not implement it as in the case of VC-1, so
+		// close and reopen the codec
+		const AVCodec *codec = CodecContext->codec;
+		avcodec_close(CodecContext);
+		// Whether or not codec is const varies between versions
+		avcodec_open2(CodecContext, const_cast<AVCodec *>(codec), 0);
 	}
 }
