@@ -26,7 +26,8 @@ extern "C" {
 #include <objbase.h>
 #endif
 
-#include <iostream> 
+#include <iostream>
+#include <fstream>
 #include <string>
 #include <stdlib.h>
 #include "ffms.h"
@@ -36,9 +37,11 @@ int TrackMask;
 int DumpMask;
 int Verbose;
 int IgnoreErrors;
+int Demuxer;
 bool Overwrite;
 bool PrintProgress;
 bool WriteTC;
+bool WriteKF;
 std::string InputFile;
 std::string CacheFile;
 std::string AudioFile;
@@ -56,10 +59,12 @@ static void PrintUsage () {
 	     << "-v        Set FFmpeg verbosity level. Can be repeated for more verbosity. (default: no messages printed)" << endl
 	     << "-p        Disable progress reporting. (default: progress reporting on)" << endl
 	     << "-c        Write timecodes for all video tracks to outputfile_track00.tc.txt (default: no)" << endl
+	     << "-k        Write keyframes for all video tracks to outputfile_track00.kf.txt (default: no)" << endl
 	     << "-t N      Set the audio indexing mask to N (-1 means index all tracks, 0 means index none, default: 0)" << endl
 	     << "-d N      Set the audio decoding mask to N (mask syntax same as -t, default: 0)" << endl
 	     << "-a NAME   Set the audio output base filename to NAME (default: input filename)" << endl
-	     << "-s N      Set audio decoding error handling. See the documentation for details. (default: 0)" << endl;
+	     << "-s N      Set audio decoding error handling. See the documentation for details. (default: 0)" << endl
+		 << "-m NAME   Force the use of demuxer NAME (default, lavf, matroska, haalimpeg, haaliogg)" << endl;
 }
 
 
@@ -76,6 +81,7 @@ static void ParseCMDLine (int argc, char *argv[]) {
 	TrackMask = 0;
 	DumpMask  = 0;
 	Verbose = 0;
+	Demuxer = FFMS_SOURCE_DEFAULT;
 	Overwrite = false;
 	IgnoreErrors = false;
 	PrintProgress = true;
@@ -97,6 +103,8 @@ static void ParseCMDLine (int argc, char *argv[]) {
 			PrintProgress = false;
 		} else if (!Option.compare("-c")) {
 			WriteTC = true;
+		} else if (!Option.compare("-k")) {
+			WriteKF = true;
 		} else if (!Option.compare("-t")) {
 			TrackMask = atoi(OptionArg.c_str());
 			i++;
@@ -108,6 +116,21 @@ static void ParseCMDLine (int argc, char *argv[]) {
 			i++;
 		} else if (!Option.compare("-s")) {
 			IgnoreErrors = atoi(OptionArg.c_str());
+			i++;
+		} else if (!Option.compare("-m")) {
+			if (!OptionArg.compare("default"))
+				Demuxer = FFMS_SOURCE_DEFAULT;
+			else if (!OptionArg.compare("lavf"))
+				Demuxer = FFMS_SOURCE_LAVF;
+			else if (!OptionArg.compare("matroska"))
+				Demuxer = FFMS_SOURCE_MATROSKA;
+			else if (!OptionArg.compare("haalimpeg"))
+				Demuxer = FFMS_SOURCE_HAALIMPEG;
+			else if (!OptionArg.compare("haaliogg"))
+				Demuxer = FFMS_SOURCE_HAALIOGG;
+			else
+				std::cout << "Warning: invalid argument to -m (" << OptionArg << "), using default instead" << std::endl;
+
 			i++;
 		} else if (InputFile.empty()) {
 			InputFile = argv[i];
@@ -152,7 +175,7 @@ static int FFMS_CC UpdateProgress(int64_t Current, int64_t Total, void *Private)
 		cout << "\b\b\b"; */
 
 	cout << "Indexing, please wait... " << Percentage << "% \r" << flush;
-	
+
 	return 0;
 }
 
@@ -176,9 +199,15 @@ static void DoIndexing () {
 
 	Index = FFMS_ReadIndex(CacheFile.c_str(), &E);
 	if (Overwrite || Index == NULL) {
-		if (PrintProgress) 
+		if (PrintProgress)
 			std::cout << "Indexing, please wait... 0% \r" << std::flush;
-		Index = FFMS_MakeIndex(InputFile.c_str(), TrackMask, DumpMask, &GenAudioFilename, NULL, IgnoreErrors, UpdateProgress, &Progress, &E);
+		FFMS_Indexer *Indexer = FFMS_CreateIndexerWithDemuxer(InputFile.c_str(), Demuxer, &E);
+		if (Indexer == NULL) {
+			std::string Err = "\nFailed to initialize indexing: ";
+			Err.append(E.Buffer);
+			throw Err;
+		}
+		Index = FFMS_DoIndexing(Indexer, TrackMask, DumpMask, &GenAudioFilename, NULL, IgnoreErrors, UpdateProgress, &Progress, &E);
 		if (Index == NULL) {
 			std::string Err = "\nIndexing error: ";
 			Err.append(E.Buffer);
@@ -207,7 +236,32 @@ static void DoIndexing () {
 			if (PrintProgress)
 				std::cout << "done." << std::endl << std::flush;
 		}
-		
+
+		if (WriteKF) {
+			if (PrintProgress)
+				std::cout << "Writing keyframes... ";
+			int NumTracks = FFMS_GetNumTracks(Index);
+			for (int t = 0; t < NumTracks; t++) {
+				FFMS_Track *Track = FFMS_GetTrackFromIndex(Index, t);
+				if (FFMS_GetTrackType(Track) == FFMS_TYPE_VIDEO && FFMS_GetNumFrames(Track)) {
+					char tn[3];
+					snprintf(tn, 3, "%02d", t);
+
+					std::ofstream kf((CacheFile + "_track" + tn + ".kf.txt").c_str());
+					kf << "# keyframe format v1" << std::endl;
+					kf << "fps 0" << std::endl;
+
+					int FrameCount = FFMS_GetNumFrames(Track);
+					for (int CurFrameNum = 0; CurFrameNum < FrameCount; CurFrameNum++) {
+						if (FFMS_GetFrameInfo(Track, CurFrameNum)->KeyFrame)
+							kf << CurFrameNum << std::endl;
+					}
+				}
+			}
+			if (PrintProgress)
+				std::cout << "done.    " << std::endl << std::flush;
+		}
+
 		if (PrintProgress)
 			std::cout << "Writing index... ";
 
@@ -226,7 +280,38 @@ static void DoIndexing () {
 }
 
 
+#ifdef _WIN32
+#ifdef __MINGW32__
+// mingw doesn't support wmain
+extern int _CRT_glob;
+extern "C" void __wgetmainargs(int*, wchar_t***, wchar_t***, int, int*);
+int main() {
+	int argc;
+	wchar_t **_argv;
+	wchar_t **env;
+	int si = 0;
+	__wgetmainargs(&argc, &_argv, &env, _CRT_glob, &si);
+#else
+int wmain(int argc, wchar_t *_argv[]) {
+#endif
+	char **argv = (char**)malloc(argc*sizeof(char*));
+	for (int i=0; i<argc; i++) {
+		int len = WideCharToMultiByte(CP_UTF8, 0, _argv[i], -1, NULL, 0, NULL, NULL);
+		if (!len) {
+			std::cout << "Failed to translate commandline to Unicode" << std::endl;
+			return 1;
+		}
+		char *temp = (char*)malloc(len*sizeof(char));
+		len = WideCharToMultiByte(CP_UTF8, 0, _argv[i], -1, temp, len, NULL, NULL);
+		if (!len) {
+			std::cout << "Failed to translate commandline to Unicode" << std::endl;
+			return 1;
+		}
+		argv[i] = temp;
+	}
+#else /* defined(_WIN32) && !defined(__MINGW32__) */
 int main(int argc, char *argv[]) {
+#endif /* defined(_WIN32) && !defined(__MINGW32__) */
 	try {
 		ParseCMDLine(argc, argv);
 	} catch (const char *Error) {
@@ -247,7 +332,7 @@ int main(int argc, char *argv[]) {
 	}
 #endif /* _WIN32 */
 
-	FFMS_Init(0);
+	FFMS_Init(0, 1);
 
 	switch (Verbose) {
 		case 0: FFMS_SetLogLevel(AV_LOG_QUIET); break;
@@ -261,19 +346,23 @@ int main(int argc, char *argv[]) {
 		DoIndexing();
 	} catch (const char *Error) {
 		std::cout << Error << std::endl;
-		FFMS_DestroyIndex(Index);
+		if (Index)
+			FFMS_DestroyIndex(Index);
 		return 1;
 	} catch (std::string Error) {
 		std::cout << std::endl << Error << std::endl;
-		FFMS_DestroyIndex(Index);
+		if (Index)
+			FFMS_DestroyIndex(Index);
 		return 1;
 	} catch (...) {
 		std::cout << std::endl << "Unknown error" << std::endl;
-		FFMS_DestroyIndex(Index);
+		if (Index)
+			FFMS_DestroyIndex(Index);
 		return 1;
 	}
 
-	FFMS_DestroyIndex(Index);
+	if (Index)
+		FFMS_DestroyIndex(Index);
 #ifdef _WIN32
 	CoUninitialize();
 #endif
